@@ -24,16 +24,16 @@
 
 /*
  * Modified by the GTK+ Team and others 2013.  See the AUTHORS
- * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * file for a list of people on the GTK Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
+ * GTK at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 /*
  * Forked from the GTK+ 3.94.0 GtkSearchBar widget and modified for libhandy by
  * Adrien Plazas on behalf of Purism SPC 2018.
  *
- * The AUTHORS file referenced above is part of GTK+ and not present in
+ * The AUTHORS file referenced above is part of GTK and not present in
  * libhandy. At the time of the fork it was available here:
  * https://gitlab.gnome.org/GNOME/gtk/blob/faba0f0145b1281facba20fb90699e3db594fbb0/AUTHORS
  *
@@ -41,13 +41,14 @@
  * the fork.
  */
 
-#include <glib/gi18n.h>
+#include "config.h"
+#include <glib/gi18n-lib.h>
 
 #include "hdy-search-bar.h"
 
 /**
  * SECTION:hdy-search-bar
- * @Short_description: A toolbar to integrate a search entry with
+ * @short_description: A toolbar to integrate a search entry with.
  * @Title: HdySearchBar
  *
  * #HdySearchBar is a container made to have a search entry (possibly
@@ -81,10 +82,13 @@ typedef struct {
   /* Template widgets */
   GtkWidget *revealer;
   GtkWidget *tool_box;
+  GtkWidget *start;
+  GtkWidget *end;
   GtkWidget *close_button;
 
   GtkWidget *entry;
   gboolean reveal_child;
+  gboolean show_close_button;
 } HdySearchBarPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (HdySearchBar, hdy_search_bar, GTK_TYPE_BIN)
@@ -98,7 +102,7 @@ enum {
 
 static GParamSpec *props[LAST_PROPERTY] = { NULL, };
 
-/* This comes from gtksearchentry.c in GTK+. */
+/* This comes from gtksearchentry.c in GTK. */
 static gboolean
 gtk_search_entry_is_keynav_event (GdkEvent *event)
 {
@@ -469,9 +473,11 @@ hdy_search_bar_class_init (HdySearchBarClass *klass)
   g_object_class_install_properties (object_class, LAST_PROPERTY, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/handy/dialer/ui/hdy-search-bar.ui");
+                                               "/sm/puri/handy/ui/hdy-search-bar.ui");
   gtk_widget_class_bind_template_child_private (widget_class, HdySearchBar, tool_box);
   gtk_widget_class_bind_template_child_private (widget_class, HdySearchBar, revealer);
+  gtk_widget_class_bind_template_child_private (widget_class, HdySearchBar, start);
+  gtk_widget_class_bind_template_child_private (widget_class, HdySearchBar, end);
   gtk_widget_class_bind_template_child_private (widget_class, HdySearchBar, close_button);
 
   gtk_widget_class_set_css_name (widget_class, "searchbar");
@@ -494,7 +500,8 @@ hdy_search_bar_init (HdySearchBar *self)
   g_signal_connect (priv->revealer, "notify::child-revealed",
                     G_CALLBACK (child_revealed_changed_cb), self);
 
-  gtk_widget_set_no_show_all (priv->close_button, TRUE);
+  gtk_widget_set_no_show_all (priv->start, TRUE);
+  gtk_widget_set_no_show_all (priv->end, TRUE);
   g_signal_connect (priv->close_button, "clicked",
                     G_CALLBACK (close_button_clicked_cb), self);
 };
@@ -622,7 +629,7 @@ hdy_search_bar_get_show_close_button (HdySearchBar *self)
 
   g_return_val_if_fail (HDY_IS_SEARCH_BAR (self), FALSE);
 
-  return gtk_widget_get_visible (priv->close_button);
+  return priv->show_close_button;
 }
 
 /**
@@ -647,8 +654,11 @@ hdy_search_bar_set_show_close_button (HdySearchBar *self,
 
   visible = visible != FALSE;
 
-  if (gtk_widget_get_visible (priv->close_button) != visible) {
-    gtk_widget_set_visible (priv->close_button, visible);
-    g_object_notify (G_OBJECT (self), "show-close-button");
-  }
+  if (priv->show_close_button == visible)
+    return;
+
+  priv->show_close_button = visible;
+  gtk_widget_set_visible (priv->start, visible);
+  gtk_widget_set_visible (priv->end, visible);
+  g_object_notify (G_OBJECT (self), "show-close-button");
 }
